#!/bin/bash
#
# mount-manager: bin/api/config-json-api.sh
#
# Require:
#   mount-manager-api.sh
#   bash-common-api:message-api.sh
#   bash-common-api:json-api.sh
#
if [ "$( type -t echofatal )" != 'function' ] ; then
  echo 'https://git.cclaude.rocks/bash-apis/bash-common-api is required: Function echofatal not found.' >&2
  exit 1
fi
if [ "$( type -t is_null_or_empty )" != 'function' ] ; then
  echo 'https://git.cclaude.rocks/bash-apis/bash-common-api is required: Function is_null_or_empty not found.' >&2
  exit 1
fi

declare -r JSON_CONFIG_FILE_DEFAULT=~/.config/mount-manager.json

declare -i CHECKLIST_HEIGHT=0
declare -i CHECKLIST_WIDTH=0
declare -i CHECKLIST_LISTHEIGHT=0

if [ -z "${MESSAGE_API_HIDE_DATE:-}" ] ; then
  declare -g MESSAGE_API_HIDE_DATE='true'
fi
export MESSAGE_API_HIDE_DATE

function _json_get_option_value {
  local -r varname="$1"

  jq ".options.${varname}" "$( json_get_config_file )"
}

function _json_get_valid_value {
  local -r value="$1"
  local -r default_value="$2"

  if [ -z "${value}" ]; then
    echo "${default_value}"
  elif [ "${value}" = "null" ]; then
    echo "${default_value}"
  else
    echo "${value}"
  fi
}

#
# API: json_get_config_file
#
function json_get_config_file {
  echo "${JSON_CONFIG_FILE_DEFAULT}"
}

function _compute_size {
  local -r json_param_name="$1"
  local -r computed_value="$2"
  local -r default_value="$3"
  local    size=

  size="$( _json_get_valid_value "$( _json_get_option_value "${json_param_name}" )" '' )"

  if [ -z "${size}" ] ; then
    size="${computed_value}"
  fi

  if [ "${size}" = "0" ] ; then
    echo "${default_value}"
  else
    echo "${size}"
  fi
}


#
# API: init_CHECKLIST_values
#
function init_CHECKLIST_values {
  local size=
  local tty_checklist_height=0
  local tty_checklist_width=0
  local json_checklist_listheight=

  if size="$( stty size 2>/dev/null )" ; then
    tty_checklist_height="$( cut -d ' ' -f1 <<<"${size}" )"
    tty_checklist_width="$( cut -d ' ' -f2 <<<"${size}" )"
  fi

  CHECKLIST_HEIGHT="$( _compute_size 'CHECKLIST_HEIGHT' "${tty_checklist_height}" 20 )"
  CHECKLIST_WIDTH="$(  _compute_size 'CHECKLIST_WIDTH'  "${tty_checklist_width}"  80 )"

  json_checklist_listheight="$( _json_get_valid_value "$( _json_get_option_value "CHECKLIST_LISTHEIGHT" )" '' )"
  if [ -z "${json_checklist_listheight}" ] ; then
    CHECKLIST_LISTHEIGHT="$(( CHECKLIST_HEIGHT - 7 ))"
  else
    CHECKLIST_LISTHEIGHT="${json_checklist_listheight}"
  fi

  echo_verbose "CHECKLIST_HEIGHT=${CHECKLIST_HEIGHT}"
  echo_verbose "CHECKLIST_WIDTH=${CHECKLIST_WIDTH}"
  echo_verbose "CHECKLIST_LISTHEIGHT=${CHECKLIST_LISTHEIGHT}"
}

function check_config { # DEPRECATED 2020-08-31
  echowarn "${FUNCNAME[0]}() is deprecated, use check_configuration instead" >&2
  check_configuration
}

#
# API: check_configuration
#
function check_configuration {
  local config_file=
  config_file="$( json_get_config_file )"

  if [ ! -f "${config_file}" ] ; then
    echofatal "ERR[2]: Configuration file is missing: '${config_file}'"
    exit 2
  fi

  # Check JSON
  jq -r '.[]' "${config_file}" >/dev/null
  local -r json_rc="$?"

  if [ "${json_rc}" -ne 0 ]; then
    echofatal "ERR[3]: Bad JSON '${config_file}' : ${json_rc}"
    exit 3
  fi

  # _check_type_ids_ 'diskimage' || exit 4 # No ID
  # _check_type_ids_ 'iscsi' || exit 5 # No ID
  _check_type_ids_ 'nfs' || exit 6
  _check_type_ids_ 'samba' || exit 7

  # TODO more checks
  check_structure || exit 100
}

function check_structure {
  local -a ids=()
  local    type_entry=
  local    id=
  local    mount_all=
  local    mount_default=
  local    error='false'

  while read -r type_entry ; do
    id="$( jq '.id' <<<"${type_entry}" )" || return 1
    mount_all="$( jq '.["mount-all"]' <<<"${type_entry}" )" || return 1
    mount_default="$( jq '.["mount-default"]' <<<"${type_entry}" )" || return 1

    if is_null_or_empty "${id}" ; then
      echofatal "id not valid: '${id}' (null or empty)"
      jq '.' <<<"${type_entry}" >&2
      error='true'
    fi
    if [ -n "$( __is_id_defined__ )" ] ; then
      echofatal "id already define: '${id}'"
      error='true'
    fi
    ids+=("${id}")

    if ! is_null_or_empty "${mount_all}" ; then
      if ! is_boolean "${mount_all}" ; then
        echofatal "mount-all not valid: '${mount_all}' - Must be true or false"
        jq '.' <<<"${type_entry}" >&2
        error='true'
      fi
    fi
    if ! is_null_or_empty "${mount_default}" ; then
      if ! is_boolean "${mount_default}" ; then
        echofatal "mount-default not valid: '${mount_default}' - Must be true or false"
        jq '.' <<<"${type_entry}" >&2
        error='true'
      fi
    fi
  done < <( _get_all_type_entries_ )
}

function __is_id_defined__ {
  local current=

  for current in "${ids[@]}" ; do
    grep "^${id}$" <<<"${current}"
  done
}

function _get_all_type_entries_ {
  local type=

  while read -r type ; do
    jq -c ".mount | .[\"${type}\"][]" "$( json_get_config_file )"
  done < <( jq -r '.mount | keys[]' "$( json_get_config_file )" )

  if [ "${error}" = 'false' ] ; then
    return 0
  else
    return 1
  fi
}

function _check_type_ids_ {
  local -r type="$1"
  local    error='false'
  local    id=

  while read -r id ; do
    if ! __is_id_valid__ "${id}" ; then
      echofatal "Bad ID found: '${id}' - Check your configuration"
      error='true'
    fi
  done < <( jq -r ".mount.${type} // [] | .[].id" "$( json_get_config_file )" )

  if [ "${error}" = 'false' ] ; then
    return 0
  else
    return 1
  fi
}

function __is_id_valid__ {
  local -r id="$1"

# ${ID}-${SHARE}
  #if [[ "${id}" = *-* ]] ; then
  #  return 1
  #el
# ${ID}/${SHARE}
  if [ -z "${id}" ] ; then
    return 1
  elif [ "${id}" = 'null' ] ; then
    return 1
  elif [[ "${id}" != "${id//[[:blank:]]/}" ]] ; then
    return 1
  else
    return 0
  fi
}

#
# API: get_type_ids
#
function get_type_ids {
  local -r type="$1"
  local    filter=
  local    id=

  if [ "${PARAM_ALL}" = 'true' ] ; then
    filter=".mount.${type} // [] | .[] | select( (.[\"mount-all\"]==true) or (.[\"mount-default\"]==true) ) | .id"
  elif [ "${PARAM_DEFAULT}" = 'true' ] ; then
    filter=".mount.${type} // [] | .[] | select( .[\"mount-default\"]==true ) | .id"
  else
    filter=".mount.${type} // [] | .[].id"
  fi

  echo_verbose "PARAM_ALL: '${PARAM_ALL}'"
  echo_verbose "PARAM_DEFAULT: '${PARAM_DEFAULT}'"
  echo_verbose "filter: '${filter}'"

  while read -r id ; do
    if __is_id_valid__ "${id}" ; then
      echo "${id}"
    else
      echofatal "Bad ID found: '${id}' for '${type}' - Check your configuration"
    fi
  done < <( jq -r "${filter}" "$( json_get_config_file )" )
}

#
# API: get_type_value type id valuename
#
function get_type_value {
  local -r type="$1"
  local -r id="$2"
  local -r valuename="$3"

  jq -r ".mount.${type} // [] | .[] | select(.id==\"${id}\").${valuename}" "$(json_get_config_file)"
}

#
# API: get_password_by_type_id TYPE ID
#
function get_password_by_type_id {
  local -r type="$1"
  local -r id="$2"
  local    gpg_pass=
  local    gpg_test=
  local    pass=

  gpg_pass="$( get_type_value "${type}" "${id}" 'gpg' )" || exit 1
  gpg_test="$( _json_get_valid_value "${gpg_pass}" 'NOTFOUND' )"

  if [ "${gpg_test}" = 'NOTFOUND' ]; then
    pass="$( get_type_value "${type}" "${id}" 'password' )" || exit 1
  else
    pass="$( decrypt "${gpg_pass}" )" || exit 1
  fi

  echo "${pass}"
}
