#!/bin/bash
#
# mount-manager:bin/api/iscsi-api.sh
#

declare -gr ISCSI_SCRIPT="${MOUNT_MANAGER_PATH}/mount-manager-iscsi.sh"

#
# API: mount_iscsi_by_mountdir mountdir
#
# mountdir : Full path of mount folder
#
function mount_iscsi_by_mountdir {
  local -r mountdir="$1"
  local iqn=
  local mountname=
  local path=
  local portal=

  mountname="$( basename "${mountdir}" )"

  echo_verbose "mount_iscsi_by_mountdir '${mountdir}'"

  portal="$( get_iscsi_portal_by_mountname "${mountname}" )"
  iqn="$( get_iscsi_iqn_by_mountname "${mountname}" )"
  path="$( get_iscsi_path_by_mountname "${mountname}" )"

  _init_ISCSI_VERBOSE_string

  echo_verbose "$( cat <<EOF
  sudo bash "${ISCSI_SCRIPT}" ${ISCSI_VERBOSE} --mount --portal "${portal}" --path "${path}" --target "${mountdir}" --iqn "${iqn}" --real-user "${REALUSER}"
EOF
)"

  # shellcheck disable=SC2086
  sudo bash "${ISCSI_SCRIPT}" ${ISCSI_VERBOSE} --mount --portal "${portal}" --path "${path}" --target "${mountdir}" --iqn "${iqn}" --real-user "${REALUSER}"
}

#
# API: umount_iscsi_by_mountdir mountdir
#
# mountdir : Full path of mount folder
#
function umount_iscsi_by_mountdir {
  local -r mountdir="$1"
  local mountname=
  local portal=

  if mount | grep "${mountdir}" --quiet; then
    mountname="$( basename "${mountdir}" )"

    echo_verbose "${FUNCNAME[0]}(): '${mountdir}'"

    portal="$( get_iscsi_portal_by_mountname "${mountname}" )"
    if [ -z "${portal}" ]; then
      echo "*** portal value not found for '${mountname}'" >&2
      exit 1
    fi

    echo_verbose "$( cat <<EOF
    sudo bash "${ISCSI_SCRIPT}" --portal "${portal}" --target "${mountdir}" --umount
EOF
    )"
    sudo bash "${ISCSI_SCRIPT}" --portal "${portal}" --target "${mountdir}" --umount
  else
    echo_verbose "'${mountdir}' not mount" >&2
  fi
}
#
# API: get_iscsi_mountdir iscsi_id
#
# iscsi_id: mountname
#
function get_iscsi_mountdir { # was get_mountdir_iscsi()
  get_mountdir "$1"
}

#
# API: mount_iscsi_by_id iscsi_id
#
# iscsi_id: mountname
#
function mount_iscsi_by_id {
  local -r mountname="$1"
  local mountdir=

  mountdir="$( get_iscsi_mountdir "${mountname}" )"

  mount_iscsi_by_mountdir "${mountdir}"
}

#
# API: mount_iscsi_by_mountdir iscsi_id
#
# iscsi_id: mountname
#
function umount_iscsi_by_id {
  local -r mountname="$1"
  local mountdir=

  mountdir="$( get_iscsi_mountdir "${mountname}" )"

  echo_verbose "umount_iscsi_by_id '${iscsi_id}' -> '${mountdir}'"
  umount_iscsi_by_mountdir "${mountdir}"
}

#
# API: mount_all_iscsi
#
function mount_all_iscsi {
  local iscsi_id=

  while read -r iscsi_id ; do
    mount_iscsi_by_id "${iscsi_id}"
  done < <( get_all_iscsi_ids )
}

#
# API: umount_all_iscsi
#
function umount_all_iscsi {
  local iscsi_id=

  for iscsi_id in $( get_all_iscsi_ids ); do
    umount_iscsi_by_id "${iscsi_id}"
  done
}

#
# API: build_checkboxes_for_iscsi
#
function build_checkboxes_for_iscsi {
  local iscsi_id=
  local mountdir=
  local mounted_status=

  for iscsi_id in $( get_all_iscsi_ids ); do
    mountdir="$( get_iscsi_mountdir "${iscsi_id}" )"
    mounted_status="$( get_mounted_status "${mountdir}" )"

    update_context_according_to_mount_status "${mounted_status}"

    CHECKBOXES+=("${mountdir}" "${iscsi_id}" "${mounted_status}")
  done
}

function _iscsi_get_portal_list {
  (
    local id=

    while read -r id ; do
      get_iscsi_portal_by_mountname "${id}"
    done < <(  get_all_iscsi_ids )
  ) | sort | uniq
}

#
# API: iscsi_logout
#
function iscsi_logout {
  local portal=

  _init_ISCSI_VERBOSE_string

  while read -r portal ; do
    echo "> iscsi logout: ${portal}"
    echo_verbose "sudo bash ${ISCSI_SCRIPT} ${ISCSI_VERBOSE} --portal \"${portal}\" --logout"
    # shellcheck disable=SC2086
    sudo bash "${ISCSI_SCRIPT}" ${ISCSI_VERBOSE} --portal "${portal}" --logout
  done < <( _iscsi_get_portal_list )
}

function _init_ISCSI_VERBOSE_string {
  if [ "${VERBOSE}" = 'true' ] ; then
    declare -g ISCSI_VERBOSE='--verbose'
  else
    declare -g ISCSI_VERBOSE=''
  fi
}
