#!/bin/bash
#
# mount-manager:bin/api/mount-manager-api.sh
#
declare -a CHECKBOXES
declare -a MOUNTED
declare -a UMOUNTED

# shellcheck disable=SC1090
source "${API_PATH}/mount-root-home-api.sh" || exit 1
# shellcheck disable=SC1090
source "${API_PATH}/diskimage-api.sh" || exit 1
# shellcheck disable=SC1090
source "${API_PATH}/iscsi-api.sh" || exit 1
# shellcheck disable=SC1090
source "${API_PATH}/iscsi-config-json-api.sh" || exit 1
# shellcheck disable=SC1090
source "${API_PATH}/nfs-api.sh" || exit 1
# shellcheck disable=SC1090
source "${API_PATH}/samba-api.sh" || exit 1

function echo_verbose {
  if [ "${VERBOSE}" = "true" ]; then
    cat <<EOF >&2
#>> $@
EOF
  fi
}

#
# API: get_mountdir mountname
#
# mountname : directory relative name's (this is the id)
#
function get_mountdir {
  local -r mountname="$1"
  local    mount_root_home=

  mount_root_home=$( get_mount_root_home ) || exit 1

  echo "${mount_root_home}/${mountname}"
}

#
# API: create_mountdir_if_needed mountdir
#
# mountdir:
#
function create_mountdir_if_needed {
  local -r mountdir="$1"

  echo_verbose "Share folder ${mountdir}"

  if [ ! -d "${mountdir}" ]; then
    echo "Create new share folder ${mountdir}"
    sudo mkdir -pv "${mountdir}"
  else
    echo_verbose "share folder already exist"
  fi

  sudo chown "${REALUSER}:${REALUSER}" "${mountdir}"
  chmod 750 "${mountdir}"
}

#
# API: umount_by_mountdir mountdir
#
# mountdir : Full path of mount folder
#
function umount_by_mountdir {
  local -r mountdir="$1"

  if mount | grep "${mountdir}" > /dev/null; then
    sudo umount -v "${mountdir}"
  else
    echo_verbose "'${mountdir}' not mount" >&2
  fi
}

function get_mounted_status {
  local -r mountdir="$1"

  if mount | grep "${mountdir}" > /dev/null; then
    echo "on"
  else
    echo "off"
  fi
}

function update_context_according_to_mount_status {
  local -r mounted_status="$1"

  if [ "${mounted_status}" = 'on' ]; then
    MOUNTED+=("${mountdir}")
  else
    UMOUNTED+=("${mountdir}")
  fi
}

function compute_display_sizes {
  local size=

  size="$( stty size )" || return 1

  CHECKLIST_HEIGHT="$( cut -d ' ' -f1 <<<"${size}" )"
  CHECKLIST_WIDTH="$( cut -d ' ' -f2 <<<"${size}" )"
  CHECKLIST_LISTHEIGHT="$(( CHECKLIST_HEIGHT - 7 ))"
}

#
# API: select_mountdirs
#
function select_mountdirs {
  CHECKBOXES=()
  MOUNTED=()
  UMOUNTED=()

  build_checkboxes_for_samba
  build_checkboxes_for_nfs
  build_checkboxes_for_iscsi
  build_checkboxes_for_diskimage

  local to_mount_list=
  local whiptail_rc=

  if [ "${#CHECKBOXES[@]}" = 0 ]; then
    echo 'Configuration file is empty or is not a valid json file.' >&2
    exit 1
  fi

  compute_display_sizes

  echo_verbose whiptail --separate-output \
    --title 'Personnal Mount Manager' \
    --checklist 'Please pick entries to mount' \
    "${CHECKLIST_HEIGHT}" "${CHECKLIST_WIDTH}" "${CHECKLIST_LISTHEIGHT}" \
    "${CHECKBOXES[@]}"

  to_mount_list=$(
    whiptail --separate-output \
    --title 'Personnal Mount Manager' \
    --checklist 'Please pick entries to mount' \
    "${CHECKLIST_HEIGHT}" "${CHECKLIST_WIDTH}" "${CHECKLIST_LISTHEIGHT}" \
    "${CHECKBOXES[@]}" 3>&1 1>&2 2>&3
    )
  whiptail_rc=$?

  echo_verbose "checklist whiptail_rc=${whiptail_rc}"

  if [ ${whiptail_rc} -eq 0 ]; then
    if [ ${#MOUNTED[@]} -ne 0 ]; then
      if [ -z "${to_mount_list}" ]; then

        # Cancel or umount all ?
        if ! whiptail --yesno "Would you like to umount all ?" --defaultno 20 60 2 ; then # not yes
          echo 'Umount all cancel by user.' >&2
          exit 1
        fi
      fi
    fi

    echo_verbose "to_mount_list=${to_mount_list}"
    [ "${#MOUNTED[@]}"  = 0 ] || echo_verbose "MOUNTED=${MOUNTED[*]}"
    [ "${#UMOUNTED[@]}" = 0 ] || echo_verbose "UMOUNTED=${UMOUNTED[*]}"
  else
    echo "Cancel by user. (${whiptail_rc})" >&2
    exit 1
  fi

  local status=
  local mountdir=
  local i=

  for ((i = 0; i < ${#UMOUNTED[@]}; i++)) ; do
    mountdir="${UMOUNTED[$i]}"

    if echo "${to_mount_list}" | grep "${mountdir}" > /dev/null; then
      status='Mounting'

      mount_by_mountdir "${mountdir}"
    else
      status='Not mounted (no change)'
    fi

    echo_verbose "[${mountdir}] ${status}"
  done

  for ((i = 0; i < ${#MOUNTED[@]}; i++)) ; do
    local mountdir="${MOUNTED[$i]}"

    if echo "${to_mount_list}" | grep "${mountdir}" > /dev/null; then
      status='Already mounted (no change)'
    else
      status='Umounting'
      umount_by_mountdir "${mountdir}"
    fi

    echo_verbose "[${mountdir}] ${status}"
  done
}

function ensure_not_empty {
  local -r var_name="$1"
  local -r var_value="${!var_name}"

  if [ -z "${var_value}" ] ; then
    echo "Missing ${var_name} value" >&2
    echo "Fix your configuration file: ${CONFIG_FILE}" >&2
    exit 1
  fi
}

function mount_all {
  mount_all_iscsi     # 1
  mount_all_nfs       # 2
  mount_all_samba     # 3

  mount_all_diskimage # 4
}

function umount_all {
  umount_all_diskimage # 4 (reverse order)

  umount_all_samba     # 3
  umount_all_nfs       # 2
  umount_all_iscsi     # 1
}

# Internal
function mount_by_mountdir {
  local -r mountdir="$1"
  local    type='samba'
  local    diskimage_mountdir=
  local    id=

  while read -r id; do
    diskimage_mountdir="$( get_diskimage_mountdir "${id}" )"

    if [ "${mountdir}" = "${diskimage_mountdir}" ]; then
      type='diskimage'
      break
    fi
  done < <( get_all_diskimage_ids )

  local iscsi_mountdir=

  while read -r id; do
    iscsi_mountdir="$( get_iscsi_mountdir "${id}" )"

    if [ "${mountdir}" = "${iscsi_mountdir}" ]; then
      type='iscsi'
      break
    fi
  done < <( get_all_iscsi_ids )

  case "${type}" in
    'samba')
      mount_samba_by_mountdir "${mountdir}"
      ;;

    'diskimage')
      mount_diskimage_by_mountdir "${mountdir}"
      ;;

    'iscsi')
      mount_iscsi_by_mountdir "${mountdir}"
      ;;

    *)
      echofatal "Internal error: type=${type}" >&2
      exit 1
      ;;
  esac
}

function __list_id__ {
  get_all_samba_ids
  get_all_nfs_ids
  get_all_iscsi_ids
  get_all_diskimage_ids
}

function list_id {
  __list_id__ | sort
}

function __list_mountdir__ {
  get_all_samba_mountdir
  get_all_nfs_mountdir
  get_all_iscsi_mountdir
  get_all_diskimage_mountdir
}

function list_mountdir {
  __list_mountdir__ | sort
}

function mountdirs {
  local mountdir=

  while [ "$#" -gt 0 ] ; do
    mountdir="$1"
    shift

    mount_by_mountdir "${mountdir}"
  done
}

function umountdirs {
  local mountdir=

  while [ "$#" -gt 0 ] ; do
    mountdir="$1"
    shift

    umount_by_mountdir "${mountdir}"
  done
}
