#!/bin/bash
#
# mount-manager:bin/api/samba-api.sh
#

# Internal
function __mount_samba__ {
  local -r username="$1"
  local -r password="$2"
  local -r server_ip="$3"
  local -r server_share="$4"
  local -r samba_id="$5" # Also server_prefix

  echo_verbose "${FUNCNAME[0]}() \"${username}\" \"*******\" \"${server_ip}\" \"${server_share}\" \"${samba_id}\""

  if [ -z "${server_ip}" ] ; then
    echofatal "${FUNCNAME[0]}() Internal error: Missing 'server_ip' value."
    exit 1
  fi
  if [ -z "${server_share}" ] ; then
    echofatal "${FUNCNAME[0]}() Internal error: Missing 'server_share' value."
    exit 1
  fi
  if [ -z "${samba_id}" ] ; then
    echofatal "${FUNCNAME[0]}() Internal error: Missing 'samba_id' value."
    exit 1
  fi

  local group_id=
  local mountdir=
  local sharepath=
  local user_id=

  mountdir="$( get_samba_mountdir "${samba_id}" "${server_share}" )"
  sharepath="$( __build_samba_sharepath__ "${server_ip}" "${server_share}" )"
  user_id="$( id --user )"
  group_id="$( id --group )"

  create_mountdir_if_needed "${mountdir}"

#  if [ -z ${username} ]; then
#    sudo mount -v -t cifs ${sharepath} -o guest,username=nobody,iocharset=utf8,gid=${group_id},uid=${user_id},sec=ntlmssp ${mountdir}
#    sudo mount -v -t cifs ${sharepath} -o guest,username=,iocharset=utf8,gid=${group_id},uid=${user_id},sec=ntlmssp ${mountdir}
#    sudo mount -v -t cifs ${sharepath} -o guest,iocharset=utf8,gid=${group_id},uid=${user_id},sec=ntlmssp ${mountdir}
#    sudo mount -v -t cifs ${sharepath} -o guest,username=${username},iocharset=utf8,gid=${group_id},uid=${user_id},sec=ntlmssp ${mountdir}
#  else
#  fi

  # -o "username=${username},password=${password},gid=${group_id},uid=${user_id},iocharset=utf8,sec=ntlmssp,vers=2.0" \
  local options="username=${username},password=${password},gid=${group_id},uid=${user_id}"
  local optional=

  #
  # https://linux.die.net/man/8/mount.cifs
  #
  optional="$( get_samba_optional_options "${samba_id}" )"
  echo_verbose "samba_id '${samba_id}' - optional '${optional}'"

  if [ "${optional}" = "null" ] || [ -z "${optional}" ] ; then
    options+=',iocharset=utf8'
    options+=',sec=ntlmssp'
    options+=',vers=2.0'
    #options+=',vers=1.0'

    # CIFS VFS: Autodisabling the use of server inode numbers.
    # This server doesn't seem to support them properly. Hardlinks will not be recognized on this mount.
    # Consider mounting with the "noserverino" option to silence this message.
    options+=',noserverino'
  else
    options+=",${optional}"
  fi

  sudo mount -v -t cifs "${sharepath}" -o "${options}" "${mountdir}"
}

#
# API: mount_samba_by_mountdir MOUNT_POINT
#
function mount_samba_by_mountdir {
  local -r mount_point="$1"
  local    mount_root_home=
  local    samba_id=
  local    dirname=
  local    share=

  mount_root_home="$( get_mount_root_home )" || exit 1

# ${ID}-${SHARE}
# dirname="$( basename "${mount_point}" )" || exit 1
#
# samba_id="$( cut -d'-' -f1 <<<"${dirname}" )" || exit 1
# share="$( cut -d'-' -f2- <<<"${dirname}" )" || exit 1
#
# if [ "$( dirname "${mount_point}" )" != "${mount_root_home}" ] ; then
#   echofatal "${FUNCNAME[0]}() : mount_point='${mount_point}' not valid."
#   exit 1
# fi

# ${ID}/${SHARE}
  share="$( basename "${mount_point}" )" || exit 1
  dirname="$( dirname "${mount_point}" )" || exit 1
  samba_id="$( basename "${dirname}" )" || exit 1

  if [ "$( dirname "${dirname}" )" != "${mount_root_home}" ] ; then
    echofatal "${FUNCNAME[0]}() : mount_point='${mount_point}' not valid."
    exit 1
  fi

  local username=
  local password=
  local ip=

  username="$( get_type_value 'samba' "${samba_id}" 'username' )" || exit 1
  password="$( get_password_by_type_id 'samba' "${samba_id}" )" || exit 1
  ip="$( get_type_value 'samba' "${samba_id}" 'ip' )" || exit 1

  __mount_samba__ "${username}" "${password}" "${ip}" "${share}" "${samba_id}"
}

# Internal
function __build_samba_sharepath__ {
  local -r server_ip="$1"
  local -r server_share="$2"

  echo "//${server_ip}/${server_share}"
}

#
# API: build_checkboxes_for_samba
#
function build_checkboxes_for_samba {
  local id=
  local ip=
  local mountdir=
  local mounted_status=
  local share=
  local share_path=

  for id in $( get_all_samba_ids ); do
    ip="$( get_samba_ip "${id}" )"

    while read -r share_name ; do

      mountdir="$( get_samba_mountdir "${id}" "${share_name}" )"
      share_path="$( __build_samba_sharepath__ "${ip}" "${share_name}" )"

      mounted_status="$( get_mounted_status "${mountdir}" )"
      update_context_according_to_mount_status "${mounted_status}"

      CHECKBOXES+=("${mountdir}" "${share_path}" "${mounted_status}")
    done < <( get_samba_shares "${id}" )
  done
}

#
# API: get_samba_mountdir SAMBA_ID SERVER_SHARE
#
function get_samba_mountdir { # was get_mountdir_samba
  local -r samba_id="$1"
  local -r server_share="$2"

# ${ID}-${SHARE}
# get_mountdir "${samba_id}-${server_share}"

# ${ID}/${SHARE}
  get_mountdir "${samba_id}/${server_share}"
}

#
# API: mount_all_samba
#
function mount_all_samba {
  local mountdir=

  while read -r mountdir ; do
    mount_samba_by_mountdir "${mountdir}"
  done < <( get_all_samba_mountdir )
}

#
# API: umount_all_samba
#
function umount_all_samba {
  local mountdir=

  while read -r mountdir ; do
    umount_by_mountdir "${mountdir}"
  done < <( get_all_samba_mountdir )
}
