#!/bin/bash
#
# mount-manager:bin/mount-manager-config.sh (Configuration helper)
#
set -u
set -o pipefail

PROJECT_HOME="$( dirname "$( dirname "$( realpath "${BASH_SOURCE[0]}" )" )" )"
declare -r PROJECT_HOME

declare -r MOUNT_MANAGER_PATH="${PROJECT_HOME}/bin"

# shellcheck disable=SC1090
source "${PROJECT_HOME}/apis/bash-common-api/bash-common-api.sh" || exit $?
# shellcheck disable=SC1090
source "${PROJECT_HOME}/apis/bash-common-api/json-api.sh" || exit $?
# shellcheck disable=SC1090
source "${MOUNT_MANAGER_PATH}/api/config-json-api.sh" || exit $?

function create_config_file {
  local -r config_file="$1"

  if [ -z "${config_file}" ]; then
    echofatal "Config file already exists: '${config_file}'."
    exit 100
  fi

  touch "${config_file}"
  chmod 600 "${config_file}" # Mandatory if plain text password are used (not recommanded)

  cat <<EOF | jq '.' >>"${config_file}"
{
"options":{"comment":"these values are optionnals","CHECKLIST_HEIGHT":20,"CHECKLIST_WIDTH":80,"CHECKLIST_LISTHEIGHT":10},
"mount":{"samba":[],"nfs":[],"diskimage":[],"iscsi":[]}
}
EOF
}

function edit_config_file {
  local -r config_file="$1"

  if [ -z "${config_file}" ]; then
    echofatal "Config file is missing."
    exit 100
  fi

  if ! is_variable_define_and_not_empty_safe EDITOR ; then
    EDITOR=''
  fi

  if [ -z "${EDITOR}" ]; then
    EDITOR="$( command -v gedit )"
  fi

  if [ -z "${EDITOR}" ]; then
    EDITOR="$( command -v xed )"
    if [ -n "${EDITOR}" ] ; then
      EDITOR+=" --new-window"
    fi
  fi

  if [ -z "${EDITOR}" ]; then
    EDITOR='vi'
  fi

  ${EDITOR} "${config_file}"
}

function check_new_configuration {
  ( check_configuration ) || return $?
}

function check_new_config_file {
  local -r config_file="$1"
  local -r config_file_lock="$2"

  if ! check_new_configuration ; then
    whiptail --yesno "New configuration file is corrupt.\n\nRestore previous version ?" 10 50
    local -i result=$?
    echo "whiptail:${result}"

    case "${result}" in
      0)
        echoinfo 'Restore previous version...'
        mv -vf "${config_file_lock}" "${config_file}" || exit 1
        ;;

      1)
        echowarn "Keep broken file: '${config_file}' (previous version is '${config_file_lock}')"
        ;;

      *)
        echofatal 'Cancel by user (keep brocken file and previous file)'
        exit 1
        ;;
    esac
  else
    local json=
    json="$( jq -c '.' "${config_file}" )" || exit 1
    jq '.' <<<"${json}" >"${config_file}"
  fi
}

function main {
  local config_file=
  local config_file_lock=

  config_file="$( json_get_config_file )"
  config_file_lock="${config_file}.lock"

  if [ ! -f "${config_file}" ]; then
    create_config_file "${config_file}" || exit $?
  elif [ ! -f "${config_file_lock}" ] ; then
    cp -v "${config_file}" "${config_file_lock}"
  else
    echofatal "Lock file is there: '${config_file_lock}'"
    echofatal 'Cancel'
    exit 1
  fi

  edit_config_file "${config_file}"
  check_new_config_file "${config_file}" "${config_file_lock}"
  if [ -f "${config_file_lock}" ] ; then
    rm -v "${config_file_lock}"
  fi
}

main "$@" || exit $?
