#!/bin/bash
#
# mount-manager:bin/mount-manager-iscsi.sh (ISCSI support helper)
#
set -u
set -o pipefail

PROJECT_HOME="$( dirname "$( dirname "$( realpath "${BASH_SOURCE[0]}" )" )" )"
declare -r PROJECT_HOME

declare -r MOUNT_MANAGER_PATH="${PROJECT_HOME}/bin"
declare -r API_PATH="${MOUNT_MANAGER_PATH}/api"

declare -g VERBOSE='false'
declare -g REALUSER=

# shellcheck disable=SC1090
source "${PROJECT_HOME}/apis/bash-common-api/bash-common-api.sh" || exit $?
# shellcheck disable=SC1090
source "${API_PATH}/mount-manager-api.sh" || exit $?

function usage {
  local -r ip_sample='192.168.1.123'
  local -r iqn_sample='iqn.2004-04.com.qnap:ts-653a:iscsi.flickr.f6d305'
  local    mount_point_sample=

  mount_point_sample=$( get_iscsi_mountdir 'iscsi-flickr' )

  cat <<EOF
Usage: $( basename "$0" ) --portal SERVER

  Actions
  -d, --discovery
  -l, --login
      --logout
  -m, --mount
      --umount

   Parameters
  -p SERVER, --portal SERVER    Name or ip of iscsi server
             --iqn IQN          Iscsi ID
             --path PATHNAME    Generated name under /dev/disk/by-path/ (optional)
             --target DIR       Expected mount point
             --password PASS    ISCSI user password
             --username USER    ISCSI user name
  --real-user USERNAME          User name

  Others
  -h, --help        Display this message and exit.
  -v, --verbose     Increase output.

  Examples:
   sudo bash $( realpath "$0" ) --portal ${ip_sample} --login
   sudo bash $( realpath "$0" ) --portal ${ip_sample} --logout
   sudo bash $( realpath "$0" ) --portal ${ip_sample} --discovery
   sudo bash $( realpath "$0" ) --portal ${ip_sample} --target ${mount_point_sample} --mount --iqn ${iqn_sample}
   sudo bash $( realpath "$0" ) --portal ${ip_sample} --target ${mount_point_sample} --umount

EOF
}

function iscsiadm_display_error_message {
  local -r  iscsiadm_cmd="$1"
  local -ri iscsiadm_rc="$2"

  case "${iscsiadm_rc}" in
    0) # OK
      echo_verbose "iscsiadm '${iscsiadm_cmd}' okay"
      ;;

    4)
      cat <<EOF >&2
*** ERROR: iscsiadm '${iscsiadm_cmd}' error: '${iscsiadm_rc}'
*** ERROR: ISCSI_ERR_TRANS (session is logged in): connect problem caused operation to fail.
EOF
      exit "${iscsiadm_rc}"
      ;;

    15)
      cat <<EOF >&2
*** ERROR: iscsiadm '${iscsiadm_cmd}' error: '${iscsiadm_rc}'
*** ERROR: ISCSI_ERR_SESS_EXISTS (session is logged in): portal '${portal_ip}' with iqn '${iqn}'
EOF
      exit "${iscsiadm_rc}"
      ;;

    21)
      cat <<EOF >&2
*** WARN: iscsiadm '${iscsiadm_cmd}' error: '${iscsiadm_rc}'
*** WARN: ISCSI_ERR_NO_OBJS_FOUND - iscsiadm login fail to portal '${portal_ip}' - try discovery.
EOF
      ;;

    *)
      cat <<EOF >&2
*** ERROR: iscsiadm '${iscsiadm_cmd}' error: '${iscsiadm_rc}'
EOF
      exit "${iscsiadm_rc}"
      ;;
  esac
}

function do_iscsiadm_full_discovery {
  local -r portal_ip="$1"

  echo_verbose "* Running: iscsiadm --mode discovery -t sendtargets --portal \"${portal_ip}\""
  iscsiadm --mode discovery -t sendtargets --portal "${portal_ip}"
  local iscsiadm_rc="$?"

  iscsiadm_display_error_message 'discovery' "${iscsiadm_rc}"

  return "${iscsiadm_rc}"
}

function do_iscsiadm_discovery {
  local -r portal_ip="$1"
  local    discovery=
  local    line=

  discovery="$(
    do_iscsiadm_full_discovery "${portal_ip}" | grep "${portal_ip}"
    )"
  local -r discovery_rc="$?"

  while IFS='' read -r line || [[ -n "${line}" ]]; do
    echo "${line}" | cut -d' ' -f2
  done <<< "${discovery}"

  return ${discovery_rc}
}

function do_iscsiadm_logout {
  local -r portal_ip="$1"

  echo_verbose "* Running: iscsiadm --mode node --portal \"${portal_ip}\" --logout"
  iscsiadm --mode node --portal "${portal_ip}" --logout
  iscsiadm_display_error_message 'logout' "$?"
}

function do_iscsiadm_login {
  local -r portal_ip="$1"
  local -r iqn="$2"
  local -r username="$3"
  local -r password="$4"

  echo_verbose "* Login: ${portal_ip} / iqn=${iqn}"

  if [ -n "${username}" ]; then
    echo_verbose "* Login with ${username} / password=${password}"

    iscsiadm --mode node --targetname "${iqn}" --portal "${portal_ip}" --op=update --name node.session.auth.authmethod --value=CHAP
    iscsiadm --mode node --targetname "${iqn}" --portal "${portal_ip}" --op=update --name node.session.auth.username --value="${username}"
    iscsiadm --mode node --targetname "${iqn}" --portal "${portal_ip}" --op=update --name node.session.auth.password --value="${password}"
  else
    echo_verbose "* iscsi annonymous login"
  fi

  echo_verbose "* Running: iscsiadm --mode node --targetname \"${iqn}\" --portal \"${portal_ip}\" --login"
  iscsiadm --mode node --targetname "${iqn}" --portal "${portal_ip}" --login
  local iscsiadm_rc=$?

  iscsiadm_display_error_message 'login' "${iscsiadm_rc}"

  if [ "${iscsiadm_rc}" -eq 21 ] ; then
    # ISCSI_ERR_NO_OBJS_FOUND - try to discover
    do_iscsiadm_login_with_discovery "${portal_ip}"
  fi

  return "${iscsiadm_rc}"
}

function do_iscsiadm_login_with_discovery {
  local -i discovery_rc=-1

  do_iscsiadm_discovery "${portal_ip}"
  discovery_rc=$?

  echo_verbose "do_iscsiadm_discovery: discovery_rc=${discovery_rc}"

  if [ "${discovery_rc}" -ne 0 ] ; then
    sleep 5
    do_iscsiadm_login "${portal_ip}" "${iqn}" "${username}" "${password}"
  else
    echo "iscsi discovery ok (${discovery_rc})" >&2
  fi
}

function find_dev {
  local -r portal_ip="$1"
  local -r iqn="$2"
  local    real_path=

  while IFS='' read -r line || [[ -n "${line}" ]]; do
    real_path="$( readlink -f "${line}" )"

    echo_verbose "${line} -> ${real_path}"
    echo "${real_path}:${line}"
  done < <( find '/dev/disk/by-path/' -type l -name "*${portal_ip}*${iqn}*" )
}

function do_mount {
  local -r portal_ip="$1"
  local -r iqn="$2"
  local -r username="$3"
  local -r password="$4"
  local -r dev_path="$5"
  local -r target="$6"
  local    login_rc=-1
  local    discovery_rc=-1

  do_iscsiadm_login "${portal_ip}" "${iqn}" "${username}" "${password}"
  login_rc=$?
  echo_verbose "* iscsiadm --login : rc=${login_rc}"

  if [ "${login_rc}" -ne 0 ] ; then
    echo "*** ERROR:${discovery_rc}/${login_rc} - iscsiadm login fail to portal '${portal_ip}' with iqn '${iqn}'." >&2
    exit "${login_rc}"
  fi

  # Create mount folder first
  create_mountdir_if_needed "${target}"

  # Let service finish to login
  sleep 3

  local devs_list=
  while [ -z "${devs_list}" ]; do
    sleep 2
    devs_list="$( find_dev "${portal_ip}" "${iqn}" )"
  done

  cat <<EOF
* -- devs_list --
${devs_list}
--
EOF

  local dev_iqn=

  if [ -z "${dev_path}" ] ; then
    case "$( wc -l <<<"${devs_list}" )"  in
      0)
        echofatal "WTF !"
        exit 100
        ;;

      1)
        dev_iqn="${devs_list}"
        ;;

      *)
        echofatal "More than one possibility. You must set --path"
        exit 1
        ;;
    esac
  else
    dev_iqn="$( grep "${dev_path}"  <<<"${devs_list}" )"
  fi

  if [ -z "${dev_iqn}" ] ; then
    echofatal "Not device found for '${dev_path}' - Check /dev/disk/by-path/"
# sudo mkfs.ext3 /dev/sdc
    exit 0
  fi

  echo_verbose "* dev_iqn=${dev_iqn}"

  local isource=
  isource="$( echo "${dev_iqn}" | cut -d':' -f1 )"

  mount --verbose --source "${isource}" --target "${target}"
  local -i mount_rc="$?"
  echo_verbose "* Running: mount --source \"${isource}\" --target \"${target}\": ${mount_rc}"

  case "${mount_rc}" in
    0)
      ;;

    32)
      echoerror "MOUNT ERROR: ${mount_rc} - (mount failure) - Is partition exist ?"
      ;;

    64)
      echoerror "MOUNT ERROR: ${mount_rc} - (some mount succeeded)" >&2
      ;;

    *)
      echofatal "MOUNT ERROR: ${mount_rc}" >&2
      exit "${mount_rc}"
      ;;
  esac
}

function do_umount {
  local -r portal_ip="$1"
  local -r mount_dir="$2"

  umount_by_mountdir "${mount_dir}"
  do_iscsiadm_logout "${portal_ip}"
}

function ensure_param_exist {
  local -r var_name="$1"
  local -r param_name="$2"
  local -r var_value="${!var_name}"

  if [ -z "${var_value}" ] ; then
    echo "Missing parameter '${param_name}'" >&2
    exit 1
  fi
}

function main {
  local action=
  local portal_ip=
  local iqn=
  local path=
  local target=
  local username=
  local password=
  local param=

  while [ "$#" -gt 0 ] ; do
    param="$1"
    shift

    case "${param}" in
      '-d'|'--discovery')
        action='discovery'
        ;;

      '-l'|'--login')
        action='login'
        ;;

      '-u'|'--logout')
        action='logout'
        ;;

      '-m'|'--mount')
        action='mount'
        ;;

      '--umount')
        action='umount'
        ;;

      '-p'|'--portal') portal_ip="$( P "$@" )" && shift ;;
      '--iqn') iqn="$( P "$@" )" && shift ;;
      '--path') path="$( P "$@" )" && shift ;;
      '--target') target="$( P "$@" )" && shift ;;
      '--password') password="$( P "$@" )" && shift ;;
      '--username') username="$( P "$@" )" && shift ;;

      '-v'|'--verbose')
        # shellcheck disable=SC2034
        VERBOSE='true'
        ;;

      '--real-user')
        REALUSER="$( P "$@" )" && shift ;;

      '-h'|'--help')
        usage
        exit 0
        ;;

      *)
        echofatal "Unknown parameter: '${param}'"
        usage >&2
        exit 1
        ;;
    esac
  done

  case "${action}" in
    'discovery')
      ensure_param_exist portal_ip '--portal'

      echo '** iscsiadm ********************************'
      do_iscsiadm_full_discovery "${portal_ip}"
      echo '**'
      echo "** iSCSI target IQN for ${portal_ip} only ********************************"
      do_iscsiadm_discovery "${portal_ip}"
      echo '**'
      echo '** devices ********************************'
      find_dev "${portal_ip}"
      echo '**'
      ;;

    'login')
      ensure_param_exist portal_ip '--portal'

      do_iscsiadm_login "${portal_ip}" || return $?
      ;;

    'logout')
      ensure_param_exist portal_ip '--portal'

      do_iscsiadm_logout "${portal_ip}" || return $?
      ;;

    'mount')
      if [ -z "${REALUSER}" ] ; then
        echofatal '--real-user is missing'
        usage >&2
        exit 1
      fi
      ensure_param_exist 'portal_ip' '--portal'
      ensure_param_exist 'iqn' '--iqn'
      ensure_param_exist 'target' '--target'

      do_mount "${portal_ip}" "${iqn}" "${username}" "${password}" "${path}" "${target}" || return $?
      ;;

    'umount')
      ensure_param_exist 'portal_ip' '--portal'
      ensure_param_exist 'target' '--target'

      do_umount "${portal_ip}" "${target}" || return $?
      ;;

    *)
      echofatal 'Missing action'
      usage >&2
      exit 1
      ;;
  esac
}

main "$@" || exit $?

